#ifndef _DESIGN_H_
#define _DESIGN_H_

/* Design Declaration - Model Branch
 *
 * Design.h
 * fwk4gps version 1.1
 * gam670/dps905
 * January 24 2011
 * copyright (c) 2011 Chris Szalwinski 
 * distributed under TPL - see ../Licenses.txt
 */

#include "iDesign.h" // for the Design Interface
#include "tlist.h"
#include "iCollisionSpace.h"

//-------------------------------- Design -------------------------------------
//
// The Design Component holds the game design proper
//
class iObject;
class iLight;
class iSound;
class iText;
class iPhysics;

class SimpleCollisionSpace;
#define MAX_CANNONBALLS 50
class Design : public iDesign {

	iContext* context;         // points to the Context object
    iScene*   scene;           // points to the Scene object
    iHUD*     hud;             // points to the HUD object
	bool fire;//used to check if the fire button has been pressed and released
	iPhysics* nf;//used to store the projectiles that get fired
	float cannonPower;//keeps track of the cannon power
	iText* powerScore;//used to put the scoring information in the HUD
	int totalScore;//keeps track of the total score
	int highestScore;//keeps track of the current high score
	bool firstUpdate;
	int lastSwitched;//used to check the time interval between switches
	bool ballSwitched,duckSwitched,katanaSwitched,bottleSwitched;//used to keep track of which object is currently in the cannon


    iLight*   topLight;        // points to the spot on the top box

    iCamera*  camera;

    int       lastUpdate;      // time that the model was last updated
    Design(iContext* c, iScene* s, iHUD* h);
    Design(const Design& s);            // prevents copying
    Design& operator=(const Design& s); // prevents assignment
    virtual ~Design() {}
    iPhysics* cannon_;
	TList<iPhysics*> flyinglist_;
	iPhysics* cannonball_;
	iPhysics* zombie; //stores the zombie mesh
	iPhysics* duck;   //stores the duck mesh
	iPhysics* monitor; //stores the monitor mesh
	iPhysics* tilted_air_wall1; 
	iPhysics* tilted_air_wall2;
	iPhysics* tilted_air_wall3;
	iPhysics* ceiling;
	iPhysics* purple_wall;
	iPhysics* orange_wall;
	iPhysics* cannonFodder; //stores the small duck that is used as a cannon ball
	iPhysics* bottleCannon; //stores the water bottle mesh
	iPhysics* ball;         //stores the ball mesh
	iPhysics* katana;       //stores the katana mesh
	iPhysics* lemon_stand;  //stores the lemonade stand mesh
	bool isFlying_[MAX_CANNONBALLS];
	iPhysics* platform_;
	int numCannonballs_;
	iPhysics* bigGreenBox_;
	iPhysics* bigBlueBox_;
	int lastfired_;                //time last time something was fired
	int lastCollision;             //the last time there was a collision
	iCollisionSpace* cs_;
public:
	friend iDesign* CreateDesign(iContext* c, iScene* s, iHUD* h);
	// initialization functions
    void   initialize(int now);
	void   reset(int now);
    void   restore(int now);
	// execution function
    void   update(int now);
    void   preDraw();
    void   draw();
	// termination functions
	void   Delete() const { delete this; }
};

#endif
